@@ -23,9 +23,13 @@ Classes
 Profile
     Represents a profile through the snapshot in Cartesian, cylindrical
     or spherical coordinates.
+Sinks
+    Represents sinks in a snapshot file.
 Snap
     Represents a smoothed particle hydrodynamics snapshot file,
     containing particles, sinks, and file header information.
+SubSnap
+    Represents a subset of particles in a snapshot file.
 Simulation
     Represents an entire smoothed particle hydrodynamics simulation.
     It contains a list of Snap objects, and time series data as pandas
@@ -50,20 +54,43 @@ See https://plonk.readthedocs.io/ for documentation. The source code is
 available at https://github.com/dmentipl/plonk.
 """
 
+import importlib_metadata
+
 from ._logging import logger_init as _logger_init
-from ._units import units
+from ._units import units, units_defaults
 from .analysis.profile import Profile, load_profile
 from .simulation.evolution import load_ev
 from .simulation.simulation import Simulation, load_sim
 from .snap.readers import load_snap
-from .snap.snap import Snap
+from .snap.snap import Sinks, Snap, SnapLike, SubSnap
 from .visualize.animation import animation, animation_particles, animation_profiles
 from .visualize.interpolation import interpolate
-from .visualize.multi import plot_snaps
-from .visualize.visualization import particle_plot, plot
+from .visualize.simulation import visualize_sim
+from .visualize.visualization import image, plot, vector
 
-# Canonical version number
-__version__ = '0.6.1'
+__version__ = importlib_metadata.version('plonk')
 
-# Initialize logger
 _logger_init(__version__)
+
+__all__ = [
+    'Profile',
+    'Simulation',
+    'Sinks',
+    'Snap',
+    'SnapLike',
+    'SubSnap',
+    'animation',
+    'animation_particles',
+    'animation_profiles',
+    'image',
+    'interpolate',
+    'load_ev',
+    'load_profile',
+    'load_sim',
+    'load_snap',
+    'plot',
+    'units',
+    'units_defaults',
+    'vector',
+    'visualize_sim',
+]
