@@ -1,30 +1,4 @@
-"""Calculate quantities on the particles.
-
-The following functions are available:
-
-- angular_momentum
-- angular_velocity
-- azimuthal_angle
-- dust_density
-- dust_fraction
-- dust_mass
-- eccentricity
-- gas_density
-- gas_fraction
-- gas_mass
-- inclination
-- keplerian_frequency
-- kinetic_energy
-- momentum
-- polar_angle
-- radial_distance
-- radial_velocity
-- semi_major_axis
-- specific_angular_momentum
-- specific_kinetic_energy
-- stokes_number
-- temperature
-"""
+"""Calculate extra quantities on the particles."""
 
 from __future__ import annotations
 
@@ -41,31 +15,7 @@ if TYPE_CHECKING:
 
 ORIGIN = (0, 0, 0) * plonk_units.au
 
-array_units = {
-    'angular_momentum': 'angular_momentum',
-    'angular_velocity': 'velocity',
-    'azimuthal_angle': 'radian',
-    'dust_density': 'density',
-    'dust_fraction': 'dimensionless',
-    'dust_mass': 'mass',
-    'eccentricity': 'dimensionless',
-    'gas_density': 'density',
-    'gas_fraction': 'dimensionless',
-    'gas_mass': 'mass',
-    'inclination': 'radian',
-    'keplerian_frequency': 'frequency',
-    'kinetic_energy': 'energy',
-    'momentum': 'momentum',
-    'polar_angle': 'radian',
-    'radial_distance': 'length',
-    'radial_velocity': 'velocity',
-    'semi_major_axis': 'length',
-    'specific_angular_momentum': 'specific_angular_momentum',
-    'specific_kinetic_energy': 'specific_energy',
-    'stokes_number': 'dimensionless',
-    'temperature': 'temperature',
-}
-
+# Derived quantities require some arrays already present
 array_requires = {
     'angular_momentum': ('mass', 'position', 'velocity'),
     'angular_velocity': ('position', 'velocity'),
@@ -91,55 +41,11 @@ array_requires = {
     'temperature': ('sound_speed',),
 }
 
-array_rotatable = {
-    'angular_momentum': True,
-    'angular_velocity': False,
-    'azimuthal_angle': False,
-    'dust_density': False,
-    'dust_fraction': False,
-    'dust_mass': False,
-    'eccentricity': False,
-    'gas_density': False,
-    'gas_fraction': False,
-    'gas_mass': False,
-    'inclination': False,
-    'keplerian_frequency': False,
-    'kinetic_energy': False,
-    'momentum': True,
-    'polar_angle': False,
-    'radial_distance': False,
-    'radial_velocity': False,
-    'semi_major_axis': False,
-    'specific_angular_momentum': True,
-    'specific_kinetic_energy': False,
-    'stokes_number': False,
-    'temperature': False,
-}
+# Arrays which represent quantities with x-, y-, z-components in space
+vector_arrays = ['angular_momentum', 'momentum', 'specific_angular_momentum']
 
-array_dust = {
-    'angular_momentum': False,
-    'angular_velocity': False,
-    'azimuthal_angle': False,
-    'dust_density': True,
-    'dust_fraction': True,
-    'dust_mass': True,
-    'eccentricity': False,
-    'gas_density': False,
-    'gas_fraction': False,
-    'gas_mass': False,
-    'inclination': False,
-    'keplerian_frequency': False,
-    'kinetic_energy': False,
-    'momentum': False,
-    'polar_angle': False,
-    'radial_distance': False,
-    'radial_velocity': False,
-    'semi_major_axis': False,
-    'specific_angular_momentum': False,
-    'specific_kinetic_energy': False,
-    'stokes_number': True,
-    'temperature': False,
-}
+# Arrays which represent dust quantities with columns per dust species
+dust_arrays = ['dust_density', 'dust_fraction', 'dust_mass', 'stokes_number']
 
 
 def momentum(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
@@ -649,8 +555,8 @@ def dust_fraction(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
         sub_type = snap['sub_type']
         type_mask = snap['type'] == snap.particle_type['dust']
         _dust_fraction = np.zeros((len(snap), n_dust))
-        for idx in range(1, n_dust + 1):
-            _dust_fraction[type_mask & (sub_type == idx), idx] = 1
+        for idx in range(n_dust):
+            _dust_fraction[type_mask & (sub_type == idx + 1), idx] = 1
 
     else:
         raise ValueError('No dust available')
@@ -686,8 +592,8 @@ def dust_mass(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
         sub_type = snap['sub_type']
         type_mask = snap['type'] == snap.particle_type['dust']
         _dust_fraction = np.zeros((len(snap), n_dust))
-        for idx in range(1, n_dust + 1):
-            _dust_fraction[type_mask & (sub_type == idx), idx] = 1
+        for idx in range(n_dust):
+            _dust_fraction[type_mask & (sub_type == idx + 1), idx] = 1
 
     else:
         raise ValueError('No dust available')
@@ -725,8 +631,8 @@ def dust_density(snap: SnapLike, ignore_accreted: bool = False) -> Quantity:
         sub_type = snap['sub_type']
         type_mask = snap['type'] == snap.particle_type['dust']
         _dust_fraction = np.zeros((len(snap), n_dust))
-        for idx in range(1, n_dust + 1):
-            _dust_fraction[type_mask & (sub_type == idx), idx] = 1
+        for idx in range(n_dust):
+            _dust_fraction[type_mask & (sub_type == idx + 1), idx] = 1
 
     else:
         raise ValueError('No dust available')
