@@ -1,7 +1,6 @@
-"""Profiles.
+"""Profiles to reduce data to 1-dimension."""
 
-Heavily inspired by pynbody (https://pynbody.github.io/).
-"""
+# Heavily inspired by pynbody (https://pynbody.github.io/).
 
 from __future__ import annotations
 
@@ -20,7 +19,7 @@ from .._logging import logger
 from .._units import Quantity
 from .._units import units as plonk_units
 from ..utils.math import average
-from ..utils.utils import is_documented_by
+from ..utils.utils import is_documented_by, pretty_array_name
 
 if TYPE_CHECKING:
     from ..snap.snap import SnapLike
@@ -77,9 +76,7 @@ class Profile:
 
     >>> prof = plonk.load_profile(snap=snap)
     >>> prof = plonk.load_profile(snap=snap, n_bins=300)
-    >>> prof = plonk.load_profile(
-    ...     snap=snap, cmin='10 au', cmax='300 au'
-    ... )
+    >>> prof = plonk.load_profile(snap=snap, cmin='10 au', cmax='300 au')
     >>> prof = plonk.load_profile(snap=snap, spacing='log')
 
     To access a profile.
@@ -101,15 +98,12 @@ class Profile:
     Plot one or many quantities on the profile.
 
     >>> prof.plot('radius', 'density')
-    >>> prof.plot(
-    ...     'radius', ['angular_momentum_x', 'angular_momentum_y'],
-    ... )
+    >>> prof.plot('radius', ['angular_momentum_x', 'angular_momentum_y'])
 
     Plot a quantity on the profile with units.
 
-    >>> prof.plot(
-    ...     'radius', 'surface_density', x_unit='au', y_unit='g/cm^2'
-    ... )
+    >>> units = {'position': 'au', 'surface_density'='g/cm^2'}
+    >>> prof.plot('radius', 'surface_density', units=units)
     """
 
     _profile_functions: Dict[str, Callable] = {}
@@ -151,7 +145,7 @@ class Profile:
         self.spacing = _check_spacing(spacing)
         self.properties: Dict[str, Any] = {}
 
-        self._profiles: Dict[str, ndarray] = {}
+        self._profiles: Dict[str, Quantity] = {}
 
         self._weights = self.snap['mass']
         self._mask = self._setup_particle_mask(ignore_accreted)
@@ -165,26 +159,29 @@ class Profile:
         self.bin_indicies = self._set_particle_bin_indicies()
 
         if ndim == 1:
-            self._profiles[coordinate] = self.bin_centers
+            self._coordinate = coordinate
         else:
-            self._profiles['radius'] = self.bin_centers
+            self._coordinate = 'radius'
+        self._profiles[self._coordinate] = self.bin_centers
         self._profiles['number'] = np.histogram(
             self._x.magnitude, self.bin_edges.magnitude
         )[0]
 
-        num_mixture_dust_species = self.snap.num_dust_species - len(
-            self.snap.num_particles_of_type['dust']
-        )
+        try:
+            num_separate_dust = len(self.snap.num_particles_of_type['dust'])
+        except KeyError:
+            num_separate_dust = 0
+        num_mixture_dust_species = self.snap.num_dust_species - num_separate_dust
         _generate_profiles(num_mixture_dust_species)
 
     def _setup_particle_mask(self, ignore_accreted: bool) -> ndarray:
         if ignore_accreted is False:
             return np.ones(len(self.snap), dtype=bool)
-        h: ndarray = self.snap['h']
+        h: Quantity = self.snap['h']
         return h > 0
 
-    def _calculate_x(self, coordinate) -> ndarray:
-        pos: ndarray = self.snap['xyz']
+    def _calculate_x(self, coordinate) -> Quantity:
+        pos: Quantity = self.snap['position']
         pos = pos[self._mask]
         if self.ndim == 1:
             if coordinate == 'x':
@@ -218,7 +215,7 @@ class Profile:
 
         return rmin, rmax
 
-    def _setup_bins(self) -> ndarray:
+    def _setup_bins(self) -> Quantity:
         bin_edges = self._bin_edges()
         if self.ndim == 1:
             bin_sizes = bin_edges[1:] - bin_edges[:-1]
@@ -249,17 +246,6 @@ class Profile:
             raise ValueError('Cannot determine spacing to setup bins')
         return bin_edges
 
-    def _bin_edges_code_units(self):
-        if self.spacing == 'linear':
-            bin_edges = np.linspace(self.range[0], self.range[1], self.n_bins + 1)
-        elif self.spacing == 'log':
-            bin_edges = np.logspace(
-                np.log10(self.range[0]), np.log10(self.range[1]), self.n_bins + 1
-            )
-        else:
-            raise ValueError('Cannot determine spacing to setup bins')
-        return bin_edges
-
     def _set_particle_bin_indicies(self) -> List[ndarray]:
         sortind = self._particle_bin.argsort()
         sort_pind = self._particle_bin[sortind]
@@ -271,7 +257,7 @@ class Profile:
             prev_index = new_index
         return binind
 
-    def _getitem(self, name: str) -> ndarray:
+    def _getitem(self, name: str) -> Quantity:
         """Return the profile of a given kind."""
         name_root = '_'.join(name.split('_')[:-1])
         name_suffix = name.split('_')[-1]
@@ -289,7 +275,7 @@ class Profile:
             aggregation = self.aggregation
             array_name = name
         try:
-            array: ndarray = self.snap[array_name]
+            array: Quantity = self.snap[array_name]
         except ValueError as e:
             logger.error(e)
             raise ValueError(f'array "{array_name}" not available on snap')
@@ -304,14 +290,14 @@ class Profile:
             'or\nprof["dust_mass_tot"].'
         )
 
-    def __getitem__(self, name: str) -> ndarray:
+    def __getitem__(self, name: str) -> Quantity:
         """Return the profile of a given kind."""
         return self._getitem(name)
 
-    def __setitem__(self, name: str, item: ndarray):
+    def __setitem__(self, name: str, item: Quantity):
         """Set the profile directly."""
-        if not isinstance(item, (ndarray, Quantity)):
-            raise ValueError('"item" must be ndarray or pint Quantity')
+        if not isinstance(item, Quantity):
+            raise ValueError('"item" must be pint Quantity')
         if item.shape[0] != self.n_bins:
             raise ValueError('Length of array does not match number of bins')
         if name in self.loaded_profiles():
@@ -346,16 +332,40 @@ class Profile:
         """Tab completion for IPython __getitem__ method."""
         return self.available_profiles()
 
-    def loaded_profiles(self):
+    def loaded_profiles(self) -> List[str]:
         """Return a listing of loaded profiles."""
-        return tuple(sorted(self._profiles.keys()))
+        return sorted(self._profiles.keys())
 
-    def available_profiles(self):
+    def available_profiles(self) -> List[str]:
         """Return a listing of available profiles."""
         loaded = list(self.loaded_profiles())
         available = list(self._profile_functions.keys())
-        snap_arrays = _1d_arrays(list(self.snap.available_arrays(all=True)))
-        return tuple(sorted(set(loaded + available + snap_arrays)))
+        snap_arrays = _1d_arrays(list(self.snap.available_arrays(verbose=True)))
+        return sorted(set(loaded + available + snap_arrays))
+
+    def base_array_name(self, name: str) -> str:
+        """Get the base array name from a string.
+
+        For example, 'velocity_x' returns 'velocity', 'density' returns
+        'density', 'dust_fraction_001' returns 'dust_fraction', 'x'
+        returns 'position'.
+
+        Parameters
+        ----------
+        name
+            The name as a string
+
+        Returns
+        -------
+        str
+            The base name.
+        """
+        try:
+            return self.snap.base_array_name(name)
+        except ValueError:
+            if name == self._coordinate:
+                return 'position'
+            return name
 
     def plot(
         self,
@@ -364,8 +374,9 @@ class Profile:
         units: Dict[str, Union[str, List[str]]] = None,
         std_dev_shading: bool = False,
         ax: Any = None,
+        ax_kwargs={},
         **kwargs,
-    ):
+    ) -> Any:
         """Plot profile.
 
         Parameters
@@ -375,15 +386,17 @@ class Profile:
         y
             The y axis to plot. Can be string or multiple as a list of
             strings.
-        units : optional
-            The units of the plot as a dictionary with keys 'x' and 'y'.
-            The values are strings representing units, e.g. 'g/cm^3'.
-            The 'y' value can be a list corresponding to each y value
-            plotted.
+        units
+            The units of the plot as a dictionary. The keys correspond to
+            quantities such as 'position', 'density', 'velocity', and so on.
+            The values are strings representing units, e.g. 'g/cm^3' for
+            density.
         std_dev_shading : optional
             Add shading for standard deviation of profile.
         ax : optional
             A matplotlib Axes object to plot to.
+        ax_kwargs
+            Keyword arguments to pass to matplotlib Axes.
         **kwargs
             Keyword arguments to pass to Axes plot method.
 
@@ -398,8 +411,10 @@ class Profile:
         if units is None:
             x_unit, y_unit = None, None
         else:
-            x_unit = units.get('x')
-            y_unit = units.get('y')
+            x_unit = units.get(self.base_array_name(x), str(self[x].units))
+            y_unit = [
+                units.get(self.base_array_name(_y), str(self[_y].units)) for _y in y
+            ]
         if y_unit is not None:
             if isinstance(y_unit, str):
                 y_unit = [y_unit for _ in range(len(y))]
@@ -413,10 +428,9 @@ class Profile:
         if ax is None:
             _, ax = plt.subplots()
 
-        xlabel = x.capitalize().replace('_', ' ')
-        xlabel = ' '.join([xlabel, f'[{_x.units:~P}]'])
+        xname = pretty_array_name(x)
+        ax.set_xlabel(f'{xname} [{_x.units:~P}]')
         _x = _x.magnitude
-        ax.set_xlabel(xlabel)
 
         for idx, yi in enumerate(y):
             _y = self[yi]
@@ -433,13 +447,13 @@ class Profile:
                     _y_mean = self[_yi + '_mean']
                 else:
                     _y_mean = _y
-            label = yi.capitalize().replace('_', ' ')
             if y_unit is not None:
                 _y = _y.to(y_unit[idx])
                 if std_dev_shading:
                     _y_std = _y_std.to(y_unit[idx])
                     _y_mean = _y_mean.to(y_unit[idx])
-            label = ' '.join([label, f'[{_y.units:~P}]'])
+            yname = pretty_array_name(yi)
+            label = f'{yname} [{_y.units:~P}]'
             _y = _y.magnitude
             if std_dev_shading:
                 _y_std = _y_std.magnitude
@@ -455,10 +469,11 @@ class Profile:
                 )
 
         ax.legend()
+        ax.set(**ax_kwargs)
 
         return ax
 
-    def to_function(self, profile, **kwargs):
+    def to_function(self, profile: str, **kwargs) -> Callable:
         """Create function via interpolation.
 
         The function is of the coordinate of the profile, e.g.
@@ -474,7 +489,8 @@ class Profile:
 
         Returns
         -------
-        The function.
+        Callable
+            The function.
 
         Examples
         --------
@@ -548,7 +564,9 @@ class Profile:
             data[name] = array
         return pd.DataFrame(data)
 
-    def particles_to_binned_quantity(self, aggregation: str, array: ndarray) -> ndarray:
+    def particles_to_binned_quantity(
+        self, aggregation: str, array: Quantity
+    ) -> Quantity:
         """Calculate binned quantities from particles.
 
         This takes care of the bin indices and ignoring accreted
@@ -592,13 +610,13 @@ class Profile:
 
 def _generate_profiles(num_mixture_dust_species: int = 0):
     @Profile.profile_property
-    def mass(prof) -> ndarray:
+    def mass(prof) -> Quantity:
         """Mass profile."""
         M = prof.snap['mass']
         return prof.particles_to_binned_quantity('sum', M)
 
     @Profile.profile_property
-    def surface_density(prof) -> ndarray:
+    def surface_density(prof) -> Quantity:
         """Surface density profile.
 
         Units are [mass / length ** ndim], which depends on ndim of profile.
@@ -606,20 +624,20 @@ def _generate_profiles(num_mixture_dust_species: int = 0):
         return prof['mass'] / prof['size']
 
     @Profile.profile_property
-    def scale_height(prof) -> ndarray:
+    def scale_height(prof) -> Quantity:
         """Scale height profile."""
         z = prof.snap['z']
         return prof.particles_to_binned_quantity('std', z)
 
     @Profile.profile_property
-    def aspect_ratio(prof) -> ndarray:
+    def aspect_ratio(prof) -> Quantity:
         """Aspect ratio profile."""
         H = prof['scale_height']
         R = prof['radius']
         return H / R
 
     @Profile.profile_property
-    def angular_momentum_theta(prof) -> ndarray:
+    def angular_momentum_theta(prof) -> Quantity:
         """Angle between specific angular momentum and xy-plane."""
         angular_momentum_z = prof['angular_momentum_z']
         angular_momentum_magnitude = prof['angular_momentum_mag']
@@ -627,14 +645,14 @@ def _generate_profiles(num_mixture_dust_species: int = 0):
         return np.arccos(angular_momentum_z / angular_momentum_magnitude)
 
     @Profile.profile_property
-    def angular_momentum_phi(prof) -> ndarray:
+    def angular_momentum_phi(prof) -> Quantity:
         """Angle between specific angular momentum and x-axis in xy-plane."""
         angular_momentum_x = prof['angular_momentum_x']
         angular_momentum_y = prof['angular_momentum_y']
         return np.arctan2(angular_momentum_y, angular_momentum_x)
 
     @Profile.profile_property
-    def toomre_Q(prof) -> ndarray:
+    def toomre_Q(prof) -> Quantity:
         """Toomre Q parameter."""
         G = (1 * plonk_units.newtonian_constant_of_gravitation).to_base_units()
         return (
@@ -646,13 +664,13 @@ def _generate_profiles(num_mixture_dust_species: int = 0):
     if num_mixture_dust_species > 0:
 
         @Profile.profile_property
-        def gas_mass(prof) -> ndarray:
+        def gas_mass(prof) -> Quantity:
             """Gas mass profile."""
             M = prof.snap['gas_mass']
             return prof.particles_to_binned_quantity('sum', M)
 
         @Profile.profile_property
-        def gas_surface_density(prof) -> ndarray:
+        def gas_surface_density(prof) -> Quantity:
             """Gas surface density profile.
 
             Units are [mass / length ** ndim], which depends on ndim of profile.
@@ -661,12 +679,12 @@ def _generate_profiles(num_mixture_dust_species: int = 0):
 
         for idx in range(num_mixture_dust_species):
 
-            def dust_mass(idx, prof) -> ndarray:
+            def dust_mass(idx, prof) -> Quantity:
                 """Dust mass profile."""
                 M = prof.snap[f'dust_mass_{idx+1:03}']
                 return prof.particles_to_binned_quantity('sum', M)
 
-            def dust_surface_density(idx, prof) -> ndarray:
+            def dust_surface_density(idx, prof) -> Quantity:
                 """Dust surface density profile.
 
                 Units are [mass / length ** ndim], which depends on ndim of profile.
