@@ -1,20 +1,4 @@
-"""Calculate global (total) quantities on the particles.
-
-The following functions are available:
-
-- center_of_mass
-- mass
-- gas_mass
-- dust_mass
-- accreted_mass
-- momentum
-- angular_momentum
-- specific_angular_momentum
-- kinetic_energy
-- specific_kinetic_energy
-- inclination
-- position_angle
-"""
+"""Calculate global (total) quantities on the particles."""
 
 from __future__ import annotations
 
