@@ -17,7 +17,8 @@ from pandas import DataFrame
 from .._logging import logger
 from .._units import Quantity
 from ..snap.readers import load_snap
-from .evolution import load_ev
+from ..visualize.simulation import visualize_sim
+from .evolution import evolution_units, load_ev
 
 if TYPE_CHECKING:
     from ..snap.snap import Snap
@@ -51,8 +52,8 @@ class Simulation:
 
     Accessing the global quantity and sink time series data.
 
-    >>> sim.global_quantities
-    >>> sim.sink_quantities
+    >>> sim.time_series['global']
+    >>> sim.time_series['sinks']
     """
 
     def __init__(self):
@@ -64,8 +65,7 @@ class Simulation:
         self._snaps: List[Snap] = None
         self._properties: Dict[str, Any] = None
         self._units: Dict[str, Any] = None
-        self._global_quantities: DataFrame = None
-        self._sink_quantities: List[DataFrame] = None
+        self._time_series: Dict[str, Union[DataFrame, List[DataFrame]]] = None
 
         self._snap_file_extension = ''
         self._len = -1
@@ -90,18 +90,12 @@ class Simulation:
             The SPH code used to produce the simulation data. Default
             is 'Phantom'.
         """
-        if not isinstance(prefix, str):
-            raise TypeError('prefix must be str')
-
         if data_source not in _data_sources:
             raise ValueError(f'Data source not available: try {_data_sources}')
         self.data_source = data_source
 
         if directory is None:
             directory = '.'
-        else:
-            if not isinstance(directory, (str, Path)):
-                raise TypeError('directory must be str or pathlib.Path')
 
         logger.debug(f'Loading {data_source} simulation: {prefix} at {directory}')
         self.prefix = prefix
@@ -115,8 +109,8 @@ class Simulation:
         self._snap_file_extension = self._get_snap_file_extension()
 
         self.paths['snaps'] = self._get_snap_files()
-        self.paths['global_quantities'] = self._get_global_ev_files()
-        self.paths['sink_quantities'] = self._get_sink_ev_files()
+        self.paths['time_series_global'] = self._get_global_ev_files()
+        self.paths['time_series_sinks'] = self._get_sink_ev_files()
 
         return self
 
@@ -145,20 +139,12 @@ class Simulation:
         return self._units
 
     @property
-    def global_quantities(self) -> DataFrame:
-        """Global quantity time series data."""
-        if self._global_quantities is None:
-            self._generate_global_quantities()
+    def time_series(self) -> DataFrame:
+        """Time series data."""
+        if self._time_series is None:
+            self._generate_time_series()
 
-        return self._global_quantities
-
-    @property
-    def sink_quantities(self) -> List[DataFrame]:
-        """Sink time series data."""
-        if self._sink_quantities is None:
-            self._generate_sink_quantities()
-
-        return self._sink_quantities
+        return self._time_series
 
     def to_array(self, quantity: str, indices: List[int] = None) -> Quantity:
         """Generate an array of a quantity over all snapshots.
@@ -186,7 +172,8 @@ class Simulation:
             (31, 1100000, 3)
         """
         q = list()
-        units = self.snaps[0][quantity].units
+        arr: Quantity = self.snaps[0][quantity]
+        units = arr.units
         for snap in self.snaps:
             if indices is None:
                 array: Quantity = snap[quantity]
@@ -230,25 +217,23 @@ class Simulation:
         self._properties = prop
 
     def _generate_units(self):
-        """Generate sim.units from snap.units."""
-        u = copy(self.snaps[0].units)
+        """Generate sim.units from snap.code_units."""
+        u = copy(self.snaps[0].code_units)
         for snap in self.snaps:
-            for key, val in snap.units.items():
+            for key, val in snap.code_units.items():
                 if u[key] != val:
                     u[key] = '__inconsistent__'
         self._units = u
 
-    def _generate_global_quantities(self):
-        """Generate global quantity time series objects."""
-        self._global_quantities = None
-        if self.paths['global_quantities']:
-            self._global_quantities = load_ev(self.paths['global_quantities'])
-
-    def _generate_sink_quantities(self):
-        """Generate sink quantity time series objects."""
-        self._sink_quantities = [
-            load_ev(files) for files in self.paths['sink_quantities']
-        ]
+    def _generate_time_series(self):
+        """Generate time series data."""
+        self._time_series = dict()
+        if self.paths['time_series_global']:
+            self._time_series['global'] = load_ev(self.paths['time_series_global'])
+        if self.paths['time_series_sinks']:
+            self._time_series['sinks'] = [
+                load_ev(files) for files in self.paths['time_series_sinks']
+            ]
 
     def _get_global_ev_files(self, glob: str = None) -> List[Path]:
         """Get global ev files."""
@@ -279,6 +264,28 @@ class Simulation:
 
         return sinks
 
+    def set_units_on_time_series(self):
+        """Set physical units on time series data."""
+        units = evolution_units(self)
+        if 'global' in self.time_series:
+            _apply_units_to_dataframe(self.time_series['global'], units)
+        if 'sinks' in self.time_series:
+            for ev in self.time_series['sinks']:
+                _apply_units_to_dataframe(ev, units)
+
+        return self
+
+    def unset_units_on_time_series(self):
+        """Un-set physical units on time series data."""
+        units = evolution_units(self)
+        if 'global' in self.time_series:
+            _un_apply_units_to_dataframe(self.time_series['global'], units)
+        if 'sinks' in self.time_series:
+            for ev in self.time_series['sinks']:
+                _un_apply_units_to_dataframe(ev, units)
+
+        return self
+
     def _get_snap_files(self, glob: str = None) -> List[Path]:
         """Get snapshot files."""
         if glob is None:
@@ -336,6 +343,8 @@ class Simulation:
             f'directory="{self.paths["directory"].name}">'
         )
 
+    visualize = visualize_sim
+
 
 def load_sim(
     prefix: str, directory: Union[str, Path] = None, data_source: str = 'Phantom',
@@ -354,6 +363,33 @@ def load_sim(
         The SPH code used to produce the simulation data. Default
         is 'Phantom'.
     """
-    return Simulation().load_sim(
-        prefix=prefix, directory=directory, data_source=data_source
+    return (
+        Simulation()
+        .load_sim(prefix=prefix, directory=directory, data_source=data_source)
+        .set_units_on_time_series()
     )
+
+
+def _apply_units_to_dataframe(dataframe, units):
+    keys = list()
+    for key, val in units.items():
+        if key in dataframe:
+            keys.append(key)
+            dataframe[key] = (dataframe[key].to_numpy() * units[key]).magnitude
+    mapper = {key: f'{key} [{units[key].units:~}]' for key in keys}
+    dataframe.rename(columns=mapper, inplace=True)
+    return dataframe
+
+
+def _un_apply_units_to_dataframe(dataframe, units):
+    keys = list()
+    for key, val in units.items():
+        key_unit = f'{key} [{units[key].units:~}]'
+        if key_unit in dataframe:
+            keys.append(key)
+            dataframe[key_unit] = (
+                dataframe[key_unit].to_numpy() / units[key]
+            ).magnitude
+    mapper = {f'{key} [{units[key].units:~}]': key for key in keys}
+    dataframe.rename(columns=mapper, inplace=True)
+    return dataframe
